;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "zzz-to-char" "20260802.944"
  "Fancy version of `zap-to-char' command."
  '((emacs "24.4")
    (avy   "0.3.0"))
  :url "https://github.com/mrkkrp/zzz-to-char"
  :commit "876cdbf60b53153bb73cabb876ab141e274b5482"
  :revdesc "876cdbf60b53"
  :keywords '("convenience")
  :authors '(("Mark Karpov" . "markkarpov92@gmail.com"))
  :maintainers '(("Mark Karpov" . "markkarpov92@gmail.com")))
